/*
Copyright 2010 Grégory Soutadé

This file is part of KissCount.

KissCount is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

KissCount is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with KissCount.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef PREFERENCESPANEL_H
#define PREFERENCESPANEL_H

#include <wx/wx.h>
#include <wx/grid.h>
#include <wx/treectrl.h>
#include <wx/statbox.h>
#include <wx/gbsizer.h>
#include <wx/stattext.h>
#include <wx/textctrl.h>
#include <wx/bmpcbox.h>

#include <controller/KissCount.h>
#include "wxUI.h"
#include <model/model.h>
#include "PasswordDialog.h"
#include "SupportedLanguages.h"

class wxUI;
class KissCount;

class PreferencesPanel: public wxPanel
{
 public:
  PreferencesPanel(KissCount* kiss, wxUI *parent);
  void ChangeUser();

  void OnAccountModified(wxGridEvent& event);
  void OnCategoryModified(wxGridEvent& event);
  void OnChangeName(wxCommandEvent& event);
  void OnChangePassword(wxCommandEvent& event);
  void OnOperationOrderChange(wxCommandEvent& event);
  void OnLanguageChange(wxCommandEvent& event);
  void OnShow(wxShowEvent& event);
  void OnKillMe(wxCommandEvent& event);

 private:
  KissCount* _kiss;
  wxUI* _wxUI;
  wxGrid* _accountsGrid;
  wxGrid* _categoriesGrid;
  wxTextCtrl* _name;
  wxBitmapComboBox* _language;
  wxComboBox* _operationOrder;

  void InitAccounts(User* user);
  void InitCategories(User* user);
  void InitLanguage(User* user);
  void InitOperationOrder(User* user);

  DECLARE_EVENT_TABLE();
};

#endif
